from dataclasses import dataclass, field
from typing import Optional

from .input_message_content import InputMessageContent
from .labeled_price import LabeledPrice


@dataclass(init=True, repr=True, slots=True)
class InputInvoiceMessageContent(InputMessageContent):
    # --- description here ---
    """Represents the [content](https://core.telegram.org/bots/api/#inputmessagecontent) of an invoice message to be sent as the result of an inline query.

    More info at: https://core.telegram.org/bots/api/#inputinvoicemessagecontent
    """

    # --- properties here ---
    title: str = field(metadata={"ac_type": [str], "ac_name": "title"})
    """Product name, 1-32 characters
    """

    description: str = field(
        metadata={"ac_type": [str], "ac_name": "description"})
    """Product description, 1-255 characters
    """

    payload: str = field(metadata={"ac_type": [str], "ac_name": "payload"})
    """Bot-defined invoice payload, 1-128 bytes. This will not be displayed to the user, use for your internal processes.
    """

    provider_token: str = field(
        metadata={"ac_type": [str], "ac_name": "provider_token"})
    """Payment provider token, obtained via [Botfather](https://t.me/botfather)
    """

    currency: str = field(metadata={"ac_type": [str], "ac_name": "currency"})
    """Three-letter ISO 4217 currency code, see [more on currencies](https://core.telegram.org/bots/payments#supported-currencies)
    """

    prices: list[LabeledPrice] = field(
        metadata={"ac_type": [LabeledPrice], "ac_name": "prices"})
    """Price breakdown, a JSON-serialized list of components (e.g. product price, tax, discount, delivery cost, delivery tax, bonus, etc.)
    """

    max_tip_amount: int = field(default=0, metadata={"ac_type": [
                                int], "ac_name": "max_tip_amount"})
    """*Optional*. The maximum accepted amount for tips in the *smallest units* of the currency (integer, **not** float/double). For example, for a maximum tip of `US$ 1.45` pass `max_tip_amount = 145`. See the *exp* parameter in [currencies.json](https://core.telegram.org/bots/payments/currencies.json), it shows the number of digits past the decimal point for each currency (2 for the majority of currencies). Defaults to 0
    """

    suggested_tip_amounts: Optional[list[int]] = field(
        default=None, metadata={"ac_type": [int], "ac_name": "suggested_tip_amounts"})
    """*Optional*. A JSON-serialized array of suggested amounts of tip in the *smallest units* of the currency (integer, **not** float/double). At most 4 suggested tip amounts can be specified. The suggested tip amounts must be positive, passed in a strictly increased order and must not exceed *max\\_tip\\_amount*.
    """

    provider_data: Optional[str] = field(
        default=None, metadata={"ac_type": [str], "ac_name": "provider_data"})
    """*Optional*. A JSON-serialized object for data about the invoice, which will be shared with the payment provider. A detailed description of the required fields should be provided by the payment provider.
    """

    photo_url: Optional[str] = field(
        default=None, metadata={"ac_type": [str], "ac_name": "photo_url"})
    """*Optional*. URL of the product photo for the invoice. Can be a photo of the goods or a marketing image for a service. People like it better when they see what they are paying for.
    """

    photo_size: Optional[int] = field(
        default=None, metadata={"ac_type": [int], "ac_name": "photo_size"})
    """*Optional*. Photo size
    """

    photo_width: Optional[int] = field(
        default=None, metadata={"ac_type": [int], "ac_name": "photo_width"})
    """*Optional*. Photo width
    """

    photo_height: Optional[int] = field(
        default=None, metadata={"ac_type": [int], "ac_name": "photo_height"})
    """*Optional*. Photo height
    """

    need_name: Optional[bool] = field(
        default=None, metadata={"ac_type": [bool], "ac_name": "need_name"})
    """*Optional*. Pass *True*, if you require the user's full name to complete the order
    """

    need_phone_number: Optional[bool] = field(
        default=None, metadata={"ac_type": [bool], "ac_name": "need_phone_number"})
    """*Optional*. Pass *True*, if you require the user's phone number to complete the order
    """

    need_email: Optional[bool] = field(
        default=None, metadata={"ac_type": [bool], "ac_name": "need_email"})
    """*Optional*. Pass *True*, if you require the user's email address to complete the order
    """

    need_shipping_address: Optional[bool] = field(
        default=None, metadata={"ac_type": [bool], "ac_name": "need_shipping_address"})
    """*Optional*. Pass *True*, if you require the user's shipping address to complete the order
    """

    send_phone_number_to_provider: Optional[bool] = field(default=None, metadata={
                                                          "ac_type": [bool], "ac_name": "send_phone_number_to_provider"})
    """*Optional*. Pass *True*, if user's phone number should be sent to provider
    """

    send_email_to_provider: Optional[bool] = field(
        default=None, metadata={"ac_type": [bool], "ac_name": "send_email_to_provider"})
    """*Optional*. Pass *True*, if user's email address should be sent to provider
    """

    is_flexible: Optional[bool] = field(
        default=None, metadata={"ac_type": [bool], "ac_name": "is_flexible"})
    """*Optional*. Pass *True*, if the final price depends on the shipping method
    """
